#include <Arduino.h>
#include <Wire.h>
#include <ClosedCube_HDC1080.h>
#include <VirtualWire.h>
#include <Rtc_Pcf8563.h>
#include <LowPower.h>

ClosedCube_HDC1080 hdc1080;

//init the real time clock
Rtc_Pcf8563 rtc;

const byte interruptPin = 2;
const byte resetPin = 3;
const byte wasserPinIn = 4;
const byte wasserPinOut = 5;
const byte ledPin = 13;

volatile int alarm_flag = 0;
char minute = 15;
long randNumber;

/* the interrupt service routine */
void blink()
{
	alarm_flag=1;
}

void setup()
{
    digitalWrite(resetPin, HIGH);
	pinMode(resetPin, OUTPUT);
    pinMode(interruptPin, INPUT_PULLUP);	// set pin to input
	pinMode(wasserPinIn, INPUT);
	pinMode(wasserPinOut, OUTPUT);
	pinMode(6, INPUT_PULLUP);
	pinMode(7, INPUT_PULLUP);
	pinMode(8, INPUT_PULLUP);
	pinMode(9, INPUT_PULLUP);
	pinMode(ledPin, OUTPUT);
	pinMode(14, INPUT_PULLUP);
	pinMode(15, INPUT_PULLUP);
	pinMode(16, INPUT_PULLUP);
	pinMode(17, INPUT_PULLUP);
	
	/* setup int on pin 2 of arduino */
	attachInterrupt(digitalPinToInterrupt(interruptPin), blink, FALLING);	

    vw_set_ptt_inverted(false);
    vw_setup(2000);     // Bits per sec (ist nur die Haelfte wegen 8 MHz)
    vw_rx_start();      // Start the receiver PLL running

	rtc.zeroClock();
	rtc.setAlarm(minute, 99, 99, 99);	// jede 15 Minuten

	hdc1080.begin(0x40);
	hdc1080.setResolution(HDC1080_RESOLUTION_14BIT, HDC1080_RESOLUTION_14BIT);
}

void loop()
{	
	char id_wasser;
	char msg[20];

	digitalWrite(wasserPinOut, HIGH);

	// Sensor auslesen
	float temp = hdc1080.readTemperature();
	float hum = hdc1080.readHumidity();

	// Wasseralarm
	if (digitalRead(wasserPinIn) == LOW)
	{
		// Wasseralarm
		id_wasser = 0x41;		// ID = 1 und Wasseralarm
	}
	else
	{
		// kein Wasseralarm
		id_wasser = 0x31;		// ID = 1
	}
	digitalWrite(wasserPinOut, LOW);

	// Werte in msg kopieren
	sprintf(msg, "%d,%02d;%d,%02d;%c", (int)temp, (int)(temp*100)%100, (int)hum, (int)(hum*100)%100, id_wasser);

	// Senden
    vw_send((uint8_t *)msg, strlen(msg));
    vw_wait_tx(); // Wait until the whole message is gone

    // falls die RTC die Interruptleitung noch nicht wieder auf HIGH gesetzt hat -> Reset
    if (digitalRead(interruptPin) == LOW)
    {
        digitalWrite(resetPin, LOW);    // Reset ausloesen
    }

	LowPower.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF);

    if (alarm_flag)
	{
		alarm_flag = 0;
		minute += 15;
		if(minute >= 60)
			minute = 0;
		rtc.setAlarm(minute, 99, 99, 99);
	}

	// Zufallswartezeit
	randNumber = random(4);
	if (randNumber > 0)
	{
		switch (randNumber)
		{
			case 1:	LowPower.powerDown(SLEEP_120MS, ADC_OFF, BOD_OFF); break;
			case 2:	LowPower.powerDown(SLEEP_250MS, ADC_OFF, BOD_OFF); break;
			case 3:	LowPower.powerDown(SLEEP_500MS, ADC_OFF, BOD_OFF); break;
			default: break;
		}
	}
}
